<?php 
namespace App\Http\Controllers;
use App\Http\Controllers\Controller;
use App\Http\Requests\ChurchAddRequest;
use App\Http\Requests\ChurchEditRequest;
use App\Models\Church;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Exception;
class ChurchController extends Controller
{
	

	/**
     * List table records
	 * @param  \Illuminate\Http\Request
     * @param string $fieldname //filter records by a table field
     * @param string $fieldvalue //filter value
     * @return \Illuminate\View\View
     */
	function index(Request $request, $fieldname = null , $fieldvalue = null){
		$view = "pages.church.list";
		$query = Church::query();
		$limit = $request->limit ?? 10;
		if($request->search){
			$search = trim($request->search);
			Church::search($query, $search); // search table records
		}
		$orderby = $request->orderby ?? "church.id";
		$ordertype = $request->ordertype ?? "desc";
		$query->orderBy($orderby, $ordertype);
		if($fieldname){
			$query->where($fieldname , $fieldvalue); //filter by a table field
		}
		$records = $query->paginate($limit, Church::listFields());
		return $this->renderView($view, compact("records"));
	}
	

	/**
     * Select table record by ID
	 * @param string $rec_id
     * @return \Illuminate\View\View
     */
	function view($rec_id = null){
		$query = Church::query();
		$record = $query->findOrFail($rec_id, Church::viewFields());
		return $this->renderView("pages.church.view", ["data" => $record]);
	}
	

	/**
     * Display form page
     * @return \Illuminate\View\View
     */
	function add(){
		return $this->renderView("pages.church.add");
	}
	

	/**
     * Save form record to the table
     * @return \Illuminate\Http\Response
     */
	function store(ChurchAddRequest $request){
		$modeldata = $this->normalizeFormData($request->validated());
		
		//save Church record
		$record = Church::create($modeldata);
		$rec_id = $record->id;
		return $this->redirect("church", "Record added successfully");
	}
	

	/**
     * Update table record with form data
	 * @param string $rec_id //select record by table primary key
     * @return \Illuminate\View\View;
     */
	function edit(ChurchEditRequest $request, $rec_id = null){
		$query = Church::query();
		$record = $query->findOrFail($rec_id, Church::editFields());
		if ($request->isMethod('post')) {
			$modeldata = $this->normalizeFormData($request->validated());
			$record->update($modeldata);
			return $this->redirect("church", "Record updated successfully");
		}
		return $this->renderView("pages.church.edit", ["data" => $record, "rec_id" => $rec_id]);
	}
	

	/**
     * Delete record from the database
	 * Support multi delete by separating record id by comma.
	 * @param  \Illuminate\Http\Request
	 * @param string $rec_id //can be separated by comma 
     * @return \Illuminate\Http\Response
     */
	function delete(Request $request, $rec_id = null){
		$arr_id = explode(",", $rec_id);
		$query = Church::query();
		$query->whereIn("id", $arr_id);
		$query->delete();
		$redirectUrl = $request->redirect ?? url()->previous();
		return $this->redirect($redirectUrl, "Record deleted successfully");
	}
    /**
     * Endpoint action
     * @return \Illuminate\Http\Response
     */
    public function appchurch(Request $request){
        $sqltext = "SELECT * FROM church";
        $records = DB::select($sqltext);
        return $records;
    }
}
