<?php 
namespace App\Http\Controllers;
use App\Http\Controllers\Controller;
use App\Http\Requests\GivingAddRequest;
use App\Models\Giving;
use Illuminate\Http\Request;
use \PDF;
use Maatwebsite\Excel\Facades\Excel;
use App\Exports\GivingListExport;
use Illuminate\Support\Facades\DB;
use Exception;
class GivingController extends Controller
{
	

	/**
     * List table records
	 * @param  \Illuminate\Http\Request
     * @param string $fieldname //filter records by a table field
     * @param string $fieldvalue //filter value
     * @return \Illuminate\View\View
     */
	function index(Request $request, $fieldname = null , $fieldvalue = null){
		$view = "pages.giving.list";
		$query = Giving::query();
		$limit = $request->limit ?? 10;
		if($request->search){
			$search = trim($request->search);
			Giving::search($query, $search); // search table records
		}
		$query->join("currency", "giving.institution", "=", "currency.cid");
		$orderby = $request->orderby ?? "giving.id";
		$ordertype = $request->ordertype ?? "desc";
		$query->orderBy($orderby, $ordertype);
		if($fieldname){
			$query->where($fieldname , $fieldvalue); //filter by a table field
		}
		// if request format is for export example:- product/index?export=pdf
		if($this->getExportFormat()){
			return $this->ExportList($query); // export current query
		}
		$records = $query->paginate($limit, Giving::listFields());
		return $this->renderView($view, compact("records"));
	}
	

	/**
     * Select table record by ID
	 * @param string $rec_id
     * @return \Illuminate\View\View
     */
	function view($rec_id = null){
		$query = Giving::query();
		$query->join("church", "giving.church_id", "=", "church.id");
		$query->join("care_group", "giving.care_group_id", "=", "care_group.id");
		$query->join("fellowship", "giving.fellowship_id", "=", "fellowship.id");
		$query->join("senior_cell", "giving.senior_cell_id", "=", "senior_cell.id");
		$query->join("cell", "giving.cell_id", "=", "cell.id");
		$record = $query->findOrFail($rec_id, Giving::viewFields());
		return $this->renderView("pages.giving.view", ["data" => $record]);
	}
	

	/**
     * Display form page
     * @return \Illuminate\View\View
     */
	function add(){
		return $this->renderView("pages.giving.add");
	}
	

	/**
     * Save form record to the table
     * @return \Illuminate\Http\Response
     */
	function store(GivingAddRequest $request){
		$modeldata = $this->normalizeFormData($request->validated());
		
		//save Giving record
		$record = Giving::create($modeldata);
		$rec_id = $record->id;
		$this->afterAdd($record);
		return $this->redirect("giving", "Record added successfully");
	}
    /**
     * After new record created
     * @param array $record // newly created record
     */
    private function afterAdd($record){
        //enter statement here
        $results = DB::select('select * from member where phone = ?', [$record->member_id]);
        foreach ($results as $p) {
            $p->full_name;
            $p->phone;
            $p->church_id;
            $p->care_group_id;
            $p->fellowship_id;
            $p->senior_cell_id;
            $p->cell_id;
            $p->email;
        }
        $give = DB::select('select * from giving where member_id = ?', [$record->member_id]);
        foreach ($give as $g) {
            $g->id;
            $g->fullname;
            $g->member_id;
            $g->amount;
            $g->institution;
            $g->datetime;
        }
        $ext = DB::select('select * from currency where cid = ?', [$g->institution]);
        foreach ($ext as $et) {
            $et->rate;
            $et->cid;
            $et->name;
        }
        DB::table('giving')->where('member_id', $record->member_id)->update(['fullname' => $p->full_name, 'church_id' => $p->church_id, 'care_group_id' => $p->care_group_id, 'fellowship_id' => $p->fellowship_id, 'senior_cell_id' => $p->senior_cell_id, 'cell_id' => $p->cell_id]);
        DB::table('giving')->where('id', $g->id)->update(['status' => $g->amount/$et->rate]);
        try{
            $subject = "CE ACCRA ROR PARTNERSHIP";
            $message = "These are your given details as at ".$g->datetime." \n";
            $message  .= "Name: ".$p->full_name." \n"; 
            $message  .= "Phone: ".$p->phone." \n"; 
            $message  .= "Amount: ".$g->amount." \n"; 
            $message  .= "Currency: ".$et->name.""; 
            $receiver = $p->email;
            $this->sendRecordActionMail($receiver, $subject, $message);
        }
        catch(Exception $ex){
            throw $ex;
        }
    }
    /**
     * Endpoint action
     * @return \Illuminate\Http\Response
     */
    public function appgiven(Request $request, $rec_id){
        $sqltext      = "SELECT * FROM giving WHERE member_id=:param1";
        $query_params = ["param1" => $rec_id];
        $records      = DB::select($sqltext, $query_params);
        return $records;
    }
	

	/**
     * Export table records to different format
	 * supported format:- PDF, CSV, EXCEL, HTML
	 * @param \Illuminate\Database\Eloquent\Model $query
     * @return \Symfony\Component\HttpFoundation\BinaryFileResponse
     */
	private function ExportList($query){
		ob_end_clean(); // clean any output to allow file download
		$filename = "ListGivingReport-" . date_now();
		$format = $this->getExportFormat();
		if($format == "print"){
			$records = $query->get(Giving::exportListFields());
			return view("reports.giving-list", ["records" => $records]);
		}
		elseif($format == "pdf"){
			$records = $query->get(Giving::exportListFields());
			$pdf = PDF::loadView("reports.giving-list", ["records" => $records]);
			return $pdf->download("$filename.pdf");
		}
		elseif($format == "csv"){
			return Excel::download(new GivingListExport($query), "$filename.csv", \Maatwebsite\Excel\Excel::CSV);
		}
		elseif($format == "excel"){
			return Excel::download(new GivingListExport($query), "$filename.xlsx", \Maatwebsite\Excel\Excel::XLSX);
		}
	}
}
