<?php 
namespace App\Http\Controllers;
use App\Http\Controllers\Controller;
use App\Http\Requests\MemberAddRequest;
use App\Http\Requests\MemberEditRequest;
use App\Models\Member;
use Illuminate\Http\Request;
use \PDF;
use Maatwebsite\Excel\Facades\Excel;
use App\Exports\MemberListExport;
use Illuminate\Support\Facades\DB;
use Exception;
class MemberController extends Controller
{
	

	/**
     * List table records
	 * @param  \Illuminate\Http\Request
     * @param string $fieldname //filter records by a table field
     * @param string $fieldvalue //filter value
     * @return \Illuminate\View\View
     */
	function index(Request $request, $fieldname = null , $fieldvalue = null){
		$view = "pages.member.list";
		$query = Member::query();
		$limit = $request->limit ?? 20;
		if($request->search){
			$search = trim($request->search);
			Member::search($query, $search); // search table records
		}
		$query->join("church", "member.church_id", "=", "church.id");
		$query->join("care_group", "member.care_group_id", "=", "care_group.id");
		$query->join("fellowship", "member.fellowship_id", "=", "fellowship.id");
		$query->join("senior_cell", "member.senior_cell_id", "=", "senior_cell.id");
		$query->join("cell", "member.cell_id", "=", "cell.id");
		$orderby = $request->orderby ?? "member.id";
		$ordertype = $request->ordertype ?? "desc";
		$query->orderBy($orderby, $ordertype);
		if($fieldname){
			$query->where($fieldname , $fieldvalue); //filter by a table field
		}
		// if request format is for export example:- product/index?export=pdf
		if($this->getExportFormat()){
			return $this->ExportList($query); // export current query
		}
		$records = $query->paginate($limit, Member::listFields());
		return $this->renderView($view, compact("records"));
	}
	

	/**
     * Select table record by ID
	 * @param string $rec_id
     * @return \Illuminate\View\View
     */
	function view($rec_id = null){
		$query = Member::query();
		$query->join("church", "member.church_id", "=", "church.id");
		$query->join("care_group", "member.care_group_id", "=", "care_group.id");
		$query->join("fellowship", "member.fellowship_id", "=", "fellowship.id");
		$query->join("senior_cell", "member.senior_cell_id", "=", "senior_cell.id");
		$query->join("cell", "member.cell_id", "=", "cell.id");
		$record = $query->findOrFail($rec_id, Member::viewFields());
		return $this->renderView("pages.member.view", ["data" => $record]);
	}
	

	/**
     * Display form page
     * @return \Illuminate\View\View
     */
	function add(){
		return $this->renderView("pages.member.add");
	}
	

	/**
     * Save form record to the table
     * @return \Illuminate\Http\Response
     */
	function store(MemberAddRequest $request){
		$modeldata = $this->normalizeFormData($request->validated());
		
		if( array_key_exists("profile_pic", $modeldata) ){
			//move uploaded file from temp directory to destination directory
			$fileInfo = $this->moveUploadedFiles($modeldata['profile_pic'], "profile_pic");
			$modeldata['profile_pic'] = $fileInfo['filepath'];
		}
		
		//save Member record
		$record = Member::create($modeldata);
		$rec_id = $record->id;
	$this->sendMailOnRecordAdd($record);
		return $this->redirect("member", "Member added successfully");
	}
	

	/**
     * Update table record with form data
	 * @param string $rec_id //select record by table primary key
     * @return \Illuminate\View\View;
     */
	function edit(MemberEditRequest $request, $rec_id = null){
		$query = Member::query();
		$record = $query->findOrFail($rec_id, Member::editFields());
		if ($request->isMethod('post')) {
			$modeldata = $this->normalizeFormData($request->validated());
		
		if( array_key_exists("profile_pic", $modeldata) ){
			//move uploaded file from temp directory to destination directory
			$fileInfo = $this->moveUploadedFiles($modeldata['profile_pic'], "profile_pic");
			$modeldata['profile_pic'] = $fileInfo['filepath'];
		}
			$record->update($modeldata);
			return $this->redirect("member", "Record updated successfully");
		}
		return $this->renderView("pages.member.edit", ["data" => $record, "rec_id" => $rec_id]);
	}
	

	/**
     * Delete record from the database
	 * Support multi delete by separating record id by comma.
	 * @param  \Illuminate\Http\Request
	 * @param string $rec_id //can be separated by comma 
     * @return \Illuminate\Http\Response
     */
	function delete(Request $request, $rec_id = null){
		$arr_id = explode(",", $rec_id);
		$query = Member::query();
		$query->whereIn("id", $arr_id);
		$query->delete();
		$redirectUrl = $request->redirect ?? url()->previous();
		return $this->redirect($redirectUrl, "Record deleted successfully");
	}
	

	/**
     * List table records
	 * @param  \Illuminate\Http\Request
     * @param string $fieldname //filter records by a table field
     * @param string $fieldvalue //filter value
     * @return \Illuminate\View\View
     */
	function awards(Request $request, $fieldname = null , $fieldvalue = null){
		$view = "pages.member.awards";
		$query = Member::query();
		$limit = $request->limit ?? 10;
		if($request->search){
			$search = trim($request->search);
			Member::search($query, $search); // search table records
		}
		$orderby = $request->orderby ?? "member.id";
		$ordertype = $request->ordertype ?? "desc";
		$query->orderBy($orderby, $ordertype);
		if($fieldname){
			$query->where($fieldname , $fieldvalue); //filter by a table field
		}
		$records = $query->paginate($limit, Member::awardsFields());
		return $this->renderView($view, compact("records"));
	}
    /**
     * Endpoint action
     * @return \Illuminate\Http\Response
     */
     public function appmember(Request $request, $rec_id){
         // $recid   = '0556804433';
         $sqltext      = "SELECT * FROM member WHERE phone=:param1";
         $query_params = ["param1" => $rec_id];
        $records      = DB::select($sqltext, $query_params);
        return $records;
    }
	

	/**
     * Export table records to different format
	 * supported format:- PDF, CSV, EXCEL, HTML
	 * @param \Illuminate\Database\Eloquent\Model $query
     * @return \Symfony\Component\HttpFoundation\BinaryFileResponse
     */
	private function ExportList($query){
		ob_end_clean(); // clean any output to allow file download
		$filename = "ListMemberReport-" . date_now();
		$format = $this->getExportFormat();
		if($format == "print"){
			$records = $query->get(Member::exportListFields());
			return view("reports.member-list", ["records" => $records]);
		}
		elseif($format == "pdf"){
			$records = $query->get(Member::exportListFields());
			$pdf = PDF::loadView("reports.member-list", ["records" => $records]);
			return $pdf->download("$filename.pdf");
		}
		elseif($format == "csv"){
			return Excel::download(new MemberListExport($query), "$filename.csv", \Maatwebsite\Excel\Excel::CSV);
		}
		elseif($format == "excel"){
			return Excel::download(new MemberListExport($query), "$filename.xlsx", \Maatwebsite\Excel\Excel::XLSX);
		}
	}
	private function sendMailOnRecordAdd($record = null){
		try{
			$subject = "New Member Account";
			$message = "Your Rhapsody Partnership App Account Successfully Created";	
			$receiver = "";
			$recid = $record->id;
			$recordLink = url("member/view/$recid");
			$this->sendRecordActionMail($receiver, $subject, $message, $recordLink);
		}
		catch(Exception $ex){
			throw $ex;
		}
	}
}
