<?php 
namespace App\Http\Controllers;
use App\Http\Controllers\Controller;
use App\Http\Requests\Senior_CellAddRequest;
use App\Http\Requests\Senior_CellEditRequest;
use App\Models\Senior_Cell;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Exception;
class Senior_CellController extends Controller
{
	

	/**
     * List table records
	 * @param  \Illuminate\Http\Request
     * @param string $fieldname //filter records by a table field
     * @param string $fieldvalue //filter value
     * @return \Illuminate\View\View
     */
	function index(Request $request, $fieldname = null , $fieldvalue = null){
		$view = "pages.senior_cell.list";
		$query = Senior_Cell::query();
		$limit = $request->limit ?? 10;
		if($request->search){
			$search = trim($request->search);
			Senior_Cell::search($query, $search); // search table records
		}
		$orderby = $request->orderby ?? "senior_cell.id";
		$ordertype = $request->ordertype ?? "desc";
		$query->orderBy($orderby, $ordertype);
		if($fieldname){
			$query->where($fieldname , $fieldvalue); //filter by a table field
		}
		$records = $query->paginate($limit, Senior_Cell::listFields());
		return $this->renderView($view, compact("records"));
	}
	

	/**
     * Select table record by ID
	 * @param string $rec_id
     * @return \Illuminate\View\View
     */
	function view($rec_id = null){
		$query = Senior_Cell::query();
		$record = $query->findOrFail($rec_id, Senior_Cell::viewFields());
		return $this->renderView("pages.senior_cell.view", ["data" => $record]);
	}
	

	/**
     * Display form page
     * @return \Illuminate\View\View
     */
	function add(){
		return $this->renderView("pages.senior_cell.add");
	}
	

	/**
     * Save form record to the table
     * @return \Illuminate\Http\Response
     */
	function store(Senior_CellAddRequest $request){
		$modeldata = $this->normalizeFormData($request->validated());
		
		//save Senior_Cell record
		$record = Senior_Cell::create($modeldata);
		$rec_id = $record->id;
		return $this->redirect("senior_cell", "Record added successfully");
	}
	

	/**
     * Update table record with form data
	 * @param string $rec_id //select record by table primary key
     * @return \Illuminate\View\View;
     */
	function edit(Senior_CellEditRequest $request, $rec_id = null){
		$query = Senior_Cell::query();
		$record = $query->findOrFail($rec_id, Senior_Cell::editFields());
		if ($request->isMethod('post')) {
			$modeldata = $this->normalizeFormData($request->validated());
			$record->update($modeldata);
			return $this->redirect("senior_cell", "Record updated successfully");
		}
		return $this->renderView("pages.senior_cell.edit", ["data" => $record, "rec_id" => $rec_id]);
	}
	

	/**
     * Delete record from the database
	 * Support multi delete by separating record id by comma.
	 * @param  \Illuminate\Http\Request
	 * @param string $rec_id //can be separated by comma 
     * @return \Illuminate\Http\Response
     */
	function delete(Request $request, $rec_id = null){
		$arr_id = explode(",", $rec_id);
		$query = Senior_Cell::query();
		$query->whereIn("id", $arr_id);
		$query->delete();
		$redirectUrl = $request->redirect ?? url()->previous();
		return $this->redirect($redirectUrl, "Record deleted successfully");
	}
	/**
     * Endpoint action
     * @return \Illuminate\Http\Response
     */
    public function appseniorcell(Request $request){
        $sqltext = "SELECT * FROM senior_cell";
        $records = DB::select($sqltext);
        return $records;
    }
}
