<?php 
namespace App\Models;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;
class Giving extends Model 
{
	

	/**
     * The table associated with the model.
     *
     * @var string
     */
	protected $table = 'giving';
	

	/**
     * The table primary key field
     *
     * @var string
     */
	protected $primaryKey = 'id';
	

	/**
     * Table fillable fields
     *
     * @var array
     */
	protected $fillable = [
		'institution','member_id','amount','transaction_no','mode_of_giving','token'
	];
	

	/**
     * Indicates if the model should be timestamped.
     *
     * @var bool
     */
	public $timestamps = false;
	

	/**
     * Set search query for the model
	 * @param \Illuminate\Database\Eloquent\Builder $query
	 * @param string $text
     */
	public static function search($query, $text){
		//search table record 
		$search_condition = '(
				giving.fullname LIKE ?  OR 
				giving.mode_of_giving LIKE ?  OR 
				giving.transaction_no LIKE ?  OR 
				giving.amount LIKE ?  OR 
				giving.institution LIKE ? 
		)';
		$search_params = [
			"%$text%","%$text%","%$text%","%$text%","%$text%"
		];
		//setting search conditions
		$query->whereRaw($search_condition, $search_params);
	}
	

	/**
     * return list page fields of the model.
     * 
     * @return array
     */
	public static function listFields(){
		return [ 
			"giving.id AS id",
			"giving.member_id AS member_id",
			"giving.fullname AS fullname",
			"giving.token AS token",
			"giving.datetime AS datetime",
			"giving.mode_of_giving AS mode_of_giving",
			"giving.transaction_no AS transaction_no",
			"giving.amount AS amount",
			"giving.institution AS institution",
			"currency.name AS currency_name",
			"giving.status AS status" 
		];
	}
	

	/**
     * return exportList page fields of the model.
     * 
     * @return array
     */
	public static function exportListFields(){
		return [ 
			"giving.id AS id",
			"giving.member_id AS member_id",
			"giving.fullname AS fullname",
			"giving.token AS token",
			"giving.datetime AS datetime",
			"giving.mode_of_giving AS mode_of_giving",
			"giving.transaction_no AS transaction_no",
			"giving.amount AS amount",
			"giving.institution AS institution",
			"currency.name AS currency_name",
			"giving.status AS status" 
		];
	}
	

	/**
     * return view page fields of the model.
     * 
     * @return array
     */
	public static function viewFields(){
		return [ 
			"giving.id AS id",
			"giving.member_id AS member_id",
			"giving.fullname AS fullname",
			"giving.church_id AS church_id",
			"church.church_name AS church_church_name",
			"giving.care_group_id AS care_group_id",
			"care_group.name AS care_group_name",
			"giving.fellowship_id AS fellowship_id",
			"fellowship.name AS fellowship_name",
			"giving.senior_cell_id AS senior_cell_id",
			"senior_cell.name AS senior_cell_name",
			"giving.cell_id AS cell_id",
			"cell.name AS cell_name",
			"giving.token AS token",
			"giving.datetime AS datetime",
			"giving.mode_of_giving AS mode_of_giving",
			"giving.transaction_no AS transaction_no",
			"giving.amount AS amount",
			"giving.institution AS institution" 
		];
	}
	

	/**
     * return exportView page fields of the model.
     * 
     * @return array
     */
	public static function exportViewFields(){
		return [ 
			"giving.id AS id",
			"giving.member_id AS member_id",
			"giving.fullname AS fullname",
			"giving.church_id AS church_id",
			"church.church_name AS church_church_name",
			"giving.care_group_id AS care_group_id",
			"care_group.name AS care_group_name",
			"giving.fellowship_id AS fellowship_id",
			"fellowship.name AS fellowship_name",
			"giving.senior_cell_id AS senior_cell_id",
			"senior_cell.name AS senior_cell_name",
			"giving.cell_id AS cell_id",
			"cell.name AS cell_name",
			"giving.token AS token",
			"giving.datetime AS datetime",
			"giving.mode_of_giving AS mode_of_giving",
			"giving.transaction_no AS transaction_no",
			"giving.amount AS amount",
			"giving.institution AS institution" 
		];
	}
}
