<?php 
namespace App\Models;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;
class Member extends Model 
{
	

	/**
     * The table associated with the model.
     *
     * @var string
     */
	protected $table = 'member';
	

	/**
     * The table primary key field
     *
     * @var string
     */
	protected $primaryKey = 'id';
	

	/**
     * Table fillable fields
     *
     * @var array
     */
	protected $fillable = [
		'full_name','email','phone','position','church_id','care_group_id','fellowship_id','senior_cell_id','cell_id','profile_pic'
	];
	

	/**
     * Indicates if the model should be timestamped.
     *
     * @var bool
     */
	public $timestamps = false;
	

	/**
     * Set search query for the model
	 * @param \Illuminate\Database\Eloquent\Builder $query
	 * @param string $text
     */
	public static function search($query, $text){
		//search table record 
		$search_condition = '(
				member.full_name LIKE ?  OR 
				member.position LIKE ?  OR 
				member.church_id LIKE ?  OR 
				member.care_group_id LIKE ?  OR 
				member.fellowship_id LIKE ?  OR 
				member.senior_cell_id LIKE ?  OR 
				member.cell_id LIKE ?  OR 
				member.phone LIKE ? 
		)';
		$search_params = [
			"%$text%","%$text%","%$text%","%$text%","%$text%","%$text%","%$text%","%$text%"
		];
		//setting search conditions
		$query->whereRaw($search_condition, $search_params);
	}
	

	/**
     * return list page fields of the model.
     * 
     * @return array
     */
	public static function listFields(){
		return [ 
			"member.id AS id",
			"member.full_name AS full_name",
			DB::raw("member.phone AS phone"),
			"member.position AS position",
			"member.church_id AS church_id",
			"church.church_name AS church_church_name",
			"member.care_group_id AS care_group_id",
			"care_group.name AS care_group_name",
			"member.fellowship_id AS fellowship_id",
			"fellowship.name AS fellowship_name",
			"member.senior_cell_id AS senior_cell_id",
			"senior_cell.name AS senior_cell_name",
			"member.cell_id AS cell_id",
			"cell.name AS cell_name",
			"member.phone AS phone" 
		];
	}
	

	/**
     * return exportList page fields of the model.
     * 
     * @return array
     */
	public static function exportListFields(){
		return [ 
			"member.id AS id",
			"member.full_name AS full_name",
			DB::raw("member.phone AS phone"),
			"member.position AS position",
			"member.church_id AS church_id",
			"church.church_name AS church_church_name",
			"member.care_group_id AS care_group_id",
			"care_group.name AS care_group_name",
			"member.fellowship_id AS fellowship_id",
			"fellowship.name AS fellowship_name",
			"member.senior_cell_id AS senior_cell_id",
			"senior_cell.name AS senior_cell_name",
			"member.cell_id AS cell_id",
			"cell.name AS cell_name",
			"member.phone AS phone" 
		];
	}
	

	/**
     * return view page fields of the model.
     * 
     * @return array
     */
	public static function viewFields(){
		return [ 
			"member.id AS id",
			"member.profile_pic AS profile_pic",
			"member.full_name AS full_name",
			"member.email AS email",
			"member.phone AS phone",
			"member.position AS position",
			"member.church_id AS church_id",
			"church.church_name AS church_church_name",
			"member.care_group_id AS care_group_id",
			"care_group.name AS care_group_name",
			"member.fellowship_id AS fellowship_id",
			"fellowship.name AS fellowship_name",
			"member.senior_cell_id AS senior_cell_id",
			"senior_cell.name AS senior_cell_name",
			"member.cell_id AS cell_id",
			"cell.name AS cell_name",
			"member.date_join AS date_join" 
		];
	}
	

	/**
     * return exportView page fields of the model.
     * 
     * @return array
     */
	public static function exportViewFields(){
		return [ 
			"member.id AS id",
			"member.profile_pic AS profile_pic",
			"member.full_name AS full_name",
			"member.email AS email",
			"member.phone AS phone",
			"member.position AS position",
			"member.church_id AS church_id",
			"church.church_name AS church_church_name",
			"member.care_group_id AS care_group_id",
			"care_group.name AS care_group_name",
			"member.fellowship_id AS fellowship_id",
			"fellowship.name AS fellowship_name",
			"member.senior_cell_id AS senior_cell_id",
			"senior_cell.name AS senior_cell_name",
			"member.cell_id AS cell_id",
			"cell.name AS cell_name",
			"member.date_join AS date_join" 
		];
	}
	

	/**
     * return edit page fields of the model.
     * 
     * @return array
     */
	public static function editFields(){
		return [ 
			"id",
			"full_name",
			"email",
			"phone",
			"position",
			"church_id",
			"care_group_id",
			"fellowship_id",
			"senior_cell_id",
			"cell_id",
			"profile_pic" 
		];
	}
	

	/**
     * return awards page fields of the model.
     * 
     * @return array
     */
	public static function awardsFields(){
		return [ 
			"id",
			"full_name",
			"email",
			"phone",
			"position",
			"amount_id",
			"user_id",
			"profile_pic",
			"date_join",
			"created_on",
			"church_id",
			"fellowship_id",
			"senior_cell_id",
			"cell_id",
			"care_group_id" 
		];
	}
	

	/**
     * return exportAwards page fields of the model.
     * 
     * @return array
     */
	public static function exportAwardsFields(){
		return [ 
			"id",
			"full_name",
			"email",
			"phone",
			"position",
			"amount_id",
			"user_id",
			"profile_pic",
			"date_join",
			"created_on",
			"church_id",
			"fellowship_id",
			"senior_cell_id",
			"cell_id",
			"care_group_id" 
		];
	}
}
