<?php
use App\Models\User;
use Illuminate\Database\Seeder;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;
use Spatie\Permission\Models\Role;
use Spatie\Permission\Models\Permission;
class RolesAndPermissionsSeeder extends Seeder
{
	private $permissionsByRole = [
			
		'administrator' => [
			'home/list', 'church/list', 'church/view', 'church/add', 'church/store', 'church/edit', 'church/delete', 'church/importdata', 'roles/list', 'roles/view', 'roles/add', 'roles/store', 'roles/edit', 'roles/delete', 'roles/importdata', 'fellowship/list', 'fellowship/view', 'fellowship/add', 'fellowship/store', 'fellowship/edit', 'fellowship/delete', 'fellowship/importdata', 'care_group/list', 'care_group/view', 'care_group/add', 'care_group/store', 'care_group/edit', 'care_group/delete', 'care_group/importdata', 'givings/importdata', 'user/list', 'user/view', 'user/add', 'user/store', 'user/edit', 'user/delete', 'user/importdata', 'senior_cell/list', 'senior_cell/view', 'senior_cell/add', 'senior_cell/store', 'senior_cell/edit', 'senior_cell/delete', 'senior_cell/importdata', 'member/list', 'member/view', 'member/add', 'member/store', 'member/edit', 'member/delete', 'member/importdata', 'cell/list', 'cell/view', 'cell/add', 'cell/store', 'cell/edit', 'cell/delete', 'cell/importdata', 'account/list', 'account/edit', 'giving/list', 'giving/view', 'giving/add', 'giving/store', 'giving/edit', 'giving/delete', 'giving/list', 'giving/view', 'giving/add', 'giving/store', 'giving/edit', 'giving/delete', 'permissions/list', 'permissions/view', 'permissions/add', 'permissions/store', 'permissions/edit', 'permissions/delete', 'model_has_permissions/list', 'model_has_permissions/view', 'model_has_permissions/add', 'model_has_permissions/store', 'model_has_permissions/edit', 'model_has_permissions/delete', 'model_has_roles/list', 'model_has_roles/view', 'model_has_roles/add', 'model_has_roles/store', 'model_has_roles/edit', 'model_has_roles/delete', 'role_has_permissions/list', 'role_has_permissions/view', 'role_has_permissions/add', 'role_has_permissions/store', 'role_has_permissions/edit', 'role_has_permissions/delete', 'church/list', 'church/view', 'church/add', 'church/store', 'church/edit', 'church/delete', 'giving/list', 'giving/view', 'giving/add', 'giving/store', 'giving/edit', 'giving/delete', 'member/awards', 'currency/list', 'currency/view', 'currency/add', 'currency/store', 'currency/edit', 'currency/delete', 'member/appmember', 'giving/appgiven', 'church/appchurch', 'care_group/appcaregroup', 'fellowship/appfellowship', 'senior_cell/appseniorcell', 'cell/appcell'
		], 
		'user' => [
			'home/list', 'account/list', 'account/edit', 'member/awards', 'member/appmember', 'giving/appgiven', 'church/appchurch', 'care_group/appcaregroup', 'fellowship/appfellowship', 'senior_cell/appseniorcell', 'cell/appcell'
		], 
		'member' => [
			'member/list', 'member/view', 'member/edit', 'giving/list', 'giving/view', 'giving/add', 'giving/store'
		]
	];
    public function run()
    {
        $tableNames = config('permission.table_names');

		Schema::disableForeignKeyConstraints();
		
		DB::table($tableNames['role_has_permissions'])->truncate();
		DB::table($tableNames['permissions'])->truncate();
        DB::table($tableNames['roles'])->truncate();
        
		Schema::enableForeignKeyConstraints();
		
		app()['cache']->forget('spatie.permission.cache');
		app()[\Spatie\Permission\PermissionRegistrar::class]->forgetCachedPermissions();

		$this->syncPermissions();
		$this->syncRoles();
		$this->syncUsersRole('administrator');
    }
	function syncRoles(){
		foreach ($this->permissionsByRole as $rolename => $permissions) {
			$role = Role::create(['name' => $rolename]);
			$role->givePermissionTo($permissions);
		}
	}

	function syncPermissions(){
		$permissions = [];

		foreach ($this->permissionsByRole as $rolename => $rolePermissions) {
			$permissions = array_merge($permissions, $rolePermissions);
		}

		$insertData = [];
		foreach($permissions as $name){
			$insertData[] = ['name'=>$name, 'guard_name' => 'web'];
		}
		Permission::insert($insertData);
	}

	function syncUsersRole($role){
		$users = User::all();
		foreach($users as $user){
			$user->syncRoles($role);
		}
	}
}
